# -*- coding: utf-8 -*-
"""
    Pontus
    ~~~~~~

    :copyright: (c) 2014 by Vesa Uimonen.
    :license: MIT, see LICENSE for more details.
"""

from .amazon_s3_file_validator import AmazonS3FileValidator  # noqa
from .amazon_s3_signed_request import AmazonS3SignedRequest  # noqa

__version__ = '1.0.0'



# -*- coding: utf-8 -*-
class ValidationError(Exception):
    def __init__(self, error):
        self.error = error

    def __str__(self):
        return 'Invalid file: {error}'.format(error=self.error)


class FileNotFoundError(Exception):
    def __init__(self, key):
        self.key = key

    def __str__(self):
        return 'File {key} was not found.'.format(key=self.key)



# -*- coding: utf-8 -*-
import magic
import re

from ._compat import force_text
from .exceptions import ValidationError


class BaseValidator(object):
    """A base class for validators used with :class:`AmazonS3FileValidator`."""
    def __call__(self, obj):
        """
        Validates an Amazon S3 file.

        :param obj: Boto S3 Object instance to be validated.
        """
        raise NotImplementedError


class MimeType(BaseValidator):
    """Validator for allowing file MIME type(s).

    Uses python-magic to determine MIME type. python-magic depends on
    libmagic library.

    Example::

        from pontus.validators import MimeType

        MimeType('image/jpeg')
        # OR
        MimeType(regex=r"...")
        # OR
        MimeType(mime_types=['image/jpeg', 'image/png'])


    :param mime_type:
        The expected MIME type.

    :param regex:
        A regular expression to match expected MIME type(s).

    :param mime_types:
        A list of expected MIME types. This will override :attr:`mime_type`
        if passed.
    """
    def __init__(self, mime_type=None, regex=None, mime_types=[]):
        if not (mime_type or regex or mime_types):
            raise ValueError(u'No argument for validation provided.')
        self.mime_types = mime_type or mime_types
        self.regex = regex

    def __call__(self, obj):
        """
        Check file MIME type is in :attr:`mime_types` or matches :attr:`regex`.

        :raises ValidationError: if the file MIME type is invalid.
        """
        file_mime_type = force_text(magic.from_buffer(
            obj.get()['Body'].read(), mime=True
        ))

        if self.regex and not re.search(self.regex, file_mime_type):
            raise ValidationError(
                u"File MIME type {mime!s} does not match r'{regex!s}'.".format(
                    mime=file_mime_type,
                    regex=self.regex
                )
            )

        if self.mime_types and (file_mime_type not in self.mime_types):
            raise ValidationError(
                (u"File MIME type is {wrong!s}, not in {right!s}.").format(
                    wrong=file_mime_type,
                    right=self.mime_types
                )
            )

    def __repr__(self):
        mime_types = (
            ' mime_types={types!r}'.format(types=self.mime_types)
            if self.mime_types else ''
        )
        regex = (
            ' regex={regex!r}'.format(regex=self.regex) if self.regex else ''
        )
        return '<{cls}{mime_types}{regex}>'.format(
            cls=self.__class__.__name__, mime_types=mime_types, regex=regex
        )


class DenyMimeType(BaseValidator):
    """Validator for denying file MIME type(s).

    Uses python-magic to determine MIME type. python-magic depends on
    libmagic library.

    Example::

        from pontus.validators import DenyMimeType

        DenyMimeType('image/jpeg')
        # OR
        DenyMimeType(regex=r"...")
        # OR
        DenyMimeType(mime_types=['image/jpeg', 'image/png'])


    :param mime_type:
        The MIME type to deny.

    :param regex:
        A regular expression to match MIME type(s) to deny.

    :param mime_types:
        A list of MIME types to deny. This will override :attr:`mime_type`
        if passed.
    """
    def __init__(self, mime_type=None, regex=None, mime_types=[]):
        if not (mime_type or regex or mime_types):
            raise ValueError(u'No argument for validation provided.')
        self.mime_types = mime_type or mime_types
        self.regex = regex

    def __call__(self, obj):
        """
        Check MIME type is not in :attr:`mime_types` or matches :attr:`regex`.

        :raises ValidationError: if the file MIME type is invalid.
        """
        file_mime_type = force_text(magic.from_buffer(
            obj.get()['Body'].read(), mime=True
        ))

        if self.regex and re.search(self.regex, file_mime_type):
            raise ValidationError(
                u"File MIME type {mime!s} matches denied regex r'{regex!s}'."
                .format(
                    mime=file_mime_type,
                    regex=self.regex
                )
            )

        if self.mime_types and (file_mime_type in self.mime_types):
            raise ValidationError(
                u"File MIME type {mime!s} is in denied list {deny_list!s}."
                .format(
                    mime=file_mime_type,
                    deny_list=self.mime_types
                )
            )

    def __repr__(self):
        mime_types = (
            ' mime_types={types!r}'.format(types=self.mime_types)
            if self.mime_types else ''
        )
        regex = (
            ' regex={regex!r}'.format(regex=self.regex) if self.regex else ''
        )
        return '<{cls}{mime_types}{regex}>'.format(
            cls=self.__class__.__name__, mime_types=mime_types, regex=regex
        )


class FileSize(BaseValidator):
    """Validator for file size.

    Example::

        from pontus.validators import FileSize

        FileSize(min=1024, max=2048)


    :param min:
        The minimum file size in bytes.

    :param max:
        The maximum file size in bytes.

    """
    def __init__(self, min=-1, max=-1):
        if not (min != -1 or max != -1):
            raise ValueError(
                u'At least one of `min` or `max` must be defined.'
            )
        if not (max == -1 or min <= max):
            raise ValueError(u'Argument `min` cannot be more than `max`.')
        self.min = min
        self.max = max

    def __call__(self, obj):
        """
        Check that the file size is between :attr:`min` and :attr:`max`.

        :raises ValidationError: if the file size is invalid.
        """
        if obj.content_length < self.min:
            raise ValidationError(u'File is smaller than %s bytes.' % self.min)
        elif self.max != -1 and obj.content_length > self.max:
            raise ValidationError(u'File is bigger than %s bytes.' % self.max)

    def __repr__(self):
        return '<{cls} min={min!r}, max={max!r}>'.format(
            cls=self.__class__.__name__,
            min=self.min,
            max=self.max
        )



# -*- coding: utf-8 -*-
import boto3
import pytest
from flask import Flask
from moto import mock_s3


@pytest.yield_fixture(scope='session', autouse=True)
def app(request):
    app = Flask('test')
    app.config.update(AWS_UNVALIDATED_PREFIX='test-unvalidated-uploads/')
    ctx = app.app_context()
    ctx.push()

    yield app

    ctx.pop()


@pytest.fixture
def bucket(request):
    boto3.resource('s3').create_bucket(Bucket='test-bucket')
    return boto3.resource('s3').Bucket('test-bucket')


@pytest.yield_fixture(autouse=True)
def mock_amazon_s3():
    with mock_s3():
        yield



# -*- coding: utf-8 -*-
import os

import pytest
import boto3

from pontus.exceptions import ValidationError
from pontus.validators import DenyMimeType


class TestDenyMimeTypeValidator(object):
    @pytest.fixture
    def jpeg_key(self, bucket):
        with open(os.path.join(
            os.path.dirname(__file__),
            'data',
            'example.jpg'
        ), 'rb') as image:
            key_name = 'example.jpg'
            obj = boto3.resource('s3').Object(bucket.name, key_name)
            obj.put(
                Body=image
            )
            return obj

    def test_raises_validation_error_if_invalid_mime_type(
        self,
        jpeg_key
    ):
        validator = DenyMimeType(mime_type='image/jpeg')
        with pytest.raises(ValidationError) as e:
            validator(jpeg_key)
        assert str(e.value) == (
            "Invalid file: File MIME type image/jpeg is in denied list "
            "image/jpeg."
        )

    def test_does_not_raise_validation_error_if_valid_mime_type(
        self,
        jpeg_key
    ):
        validator = DenyMimeType(mime_type='image/png')
        validator(jpeg_key)

    def test_repr(self):
        assert repr(DenyMimeType(mime_type='image/png')) == (
            u"<DenyMimeType mime_types='image/png'>"
        )

    def test_raises_validation_error_if_mime_type_not_in_valid_mime_types(
        self,
        jpeg_key
    ):
        validator = DenyMimeType(mime_types=['image/jpeg', 'application/csv'])
        with pytest.raises(ValidationError) as e:
            validator(jpeg_key)
        assert str(e.value) == (
            "Invalid file: File MIME type image/jpeg is in denied list "
            "['image/jpeg', 'application/csv']."
        )

    def test_doesnt_raise_validation_error_if_mime_type_in_valid_mime_types(
        self,
        jpeg_key
    ):
        validator = DenyMimeType(mime_types=['image/png', 'application/csv'])
        validator(jpeg_key)

    def test_raises_validation_error_if_mime_type_doesnt_match_regex(
        self,
        jpeg_key
    ):
        validator = DenyMimeType(regex=r'image\/.*')
        with pytest.raises(ValidationError) as e:
            validator(jpeg_key)
        assert str(e.value) == (
            "Invalid file: File MIME type image/jpeg matches denied regex "
            "r'image\/.*'."
        )

    def test_doesnt_raise_validation_error_if_mime_type_matches_regex(
        self,
        jpeg_key
    ):
        validator = DenyMimeType(regex=r'application\/.*')
        validator(jpeg_key)



# -*- coding: utf-8 -*-
import pytest
import botocore
import boto3
from flexmock import flexmock

from pontus import AmazonS3FileValidator
from pontus.exceptions import FileNotFoundError, ValidationError
from pontus.validators import BaseValidator

HOUR_IN_SECONDS = 60 * 60


class CustomValidator(BaseValidator):
    def __call__(self, obj):
        if obj.key != 'test-unvalidated-uploads/images/hello.jpg':
            raise ValidationError('Invalid.')


class TestAmazonS3FileValidator(object):
    @pytest.fixture
    def amazon_s3_file_validator(self, bucket):
        key_name = 'test-unvalidated-uploads/images/hello.jpg'
        boto3.resource('s3').Object(bucket.name, key_name).put(Body='test')
        return AmazonS3FileValidator(
            key_name=key_name,
            bucket=bucket,
            validators=[CustomValidator()]
        )

    @pytest.fixture
    def failing_amazon_s3_file_validator(self, bucket):
        key_name = 'images/fail.jpg'
        boto3.resource('s3').Object(bucket.name, key_name).put(Body='test')
        return AmazonS3FileValidator(
            key_name=key_name,
            bucket=bucket,
            validators=[CustomValidator()]
        )

    def test_validate_calls_validators(self, amazon_s3_file_validator):
        (
            flexmock(CustomValidator)
            .should_receive('__call__')
            .with_args(amazon_s3_file_validator.obj)
            .and_return(True)
        )
        amazon_s3_file_validator.validate()

    def test_validate_returns_true_when_validation_passes(
        self,
        amazon_s3_file_validator
    ):
        assert amazon_s3_file_validator.validate()

    def test_validator_has_no_errors_when_validation_passes(
        self,
        amazon_s3_file_validator
    ):
        assert not amazon_s3_file_validator.errors

    def test_validate_returns_false_when_validation_fails(
        self,
        failing_amazon_s3_file_validator
    ):
        assert not failing_amazon_s3_file_validator.validate()

    def test_validator_has_errors_when_validation_fails(
        self,
        failing_amazon_s3_file_validator
    ):
        failing_amazon_s3_file_validator.validate()
        assert failing_amazon_s3_file_validator.errors == ['Invalid.']

    def test_throws_error_if_file_not_found(
        self,
        bucket
    ):
        with pytest.raises(FileNotFoundError):
            AmazonS3FileValidator(
                key_name='does_not_exist.jpg',
                bucket=bucket
            )

    def test_validate_removes_unvalidated_prefix_if_validation_passes(
        self,
        amazon_s3_file_validator,
        bucket
    ):
        amazon_s3_file_validator.validate()

        with pytest.raises(botocore.exceptions.ClientError):
            boto3.resource('s3').Object(
              bucket.name, 'test-unvalidated-uploads/images/hello.jpg'
            ).get()

        assert boto3.resource('s3').Object(
              bucket.name, 'images/hello.jpg'
        ).get()
        assert not amazon_s3_file_validator.obj.key.startswith(
            'test-unvalidated-uploads/'
        )

    def test_repr(self, amazon_s3_file_validator):
        assert repr(amazon_s3_file_validator) == (
            "<AmazonS3FileValidator " +
            "key='test-unvalidated-uploads/images/hello.jpg'>"
        )



# -*- coding: utf-8 -*-
import os

import pytest
import boto3

from pontus.exceptions import ValidationError
from pontus.validators import FileSize


class TestFileSizeValidator(object):
    @pytest.fixture
    def jpeg_key(self, bucket):
        with open(os.path.join(
            os.path.dirname(__file__),
            'data',
            'example.jpg'
        ), 'rb') as image:
            key_name = 'example.jpg'
            obj = boto3.resource('s3').Object(bucket.name, key_name)
            obj.put(
                Body=image
            )
            return obj

    def test_raises_validation_error_if_file_is_too_large(
        self,
        jpeg_key
    ):
        validator = FileSize(max=27660)
        with pytest.raises(ValidationError) as e:
            validator(jpeg_key)
        assert e.value.error == (
            u'File is bigger than 27660 bytes.'
        )

    def test_raises_validation_error_if_file_is_too_small(
        self,
        jpeg_key
    ):
        validator = FileSize(min=27670)
        with pytest.raises(ValidationError) as e:
            validator(jpeg_key)
        assert e.value.error == (
            u'File is smaller than 27670 bytes.'
        )

    def test_does_not_raise_validation_error_if_file_is_of_valid_size(
        self,
        jpeg_key
    ):
        validator = FileSize(min=27660, max=27662)
        validator(jpeg_key)

    def test_raises_value_error_if_no_min_or_max_given(self):
        with pytest.raises(ValueError) as e:
            FileSize()
        assert str(e.value) == (
            'At least one of `min` or `max` must be defined.'
        )

    def test_raises_value_error_if_min_is_more_than_max(self):
        with pytest.raises(ValueError) as e:
            FileSize(min=2, max=1)
        assert str(e.value) == (
            'Argument `min` cannot be more than `max`.'
        )

    def test_repr(self):
        assert repr(FileSize(min=27660, max=27662)) == (
            u"<FileSize min=27660, max=27662>"
        )



"""
Pontus
------

Flask utility for signing Amazon S3 POST requests and validating Amazon S3
files.
"""

import os
import re
import subprocess

from setuptools import Command, setup


def get_version():
    filename = os.path.join(
        os.path.dirname(os.path.abspath(__file__)),
        'pontus',
        '__init__.py'
    )
    contents = open(filename).read()
    pattern = r"^__version__ = '(.*?)'$"
    return re.search(pattern, contents, re.MULTILINE).group(1)


class PyTest(Command):
    user_options = []

    def initialize_options(self):
        pass

    def finalize_options(self):
        pass

    def run(self):
        errno = subprocess.call(['py.test'])
        raise SystemExit(errno)


extras_require = {
    'test': [
        'flexmock>=0.9.7',
        'freezegun>=0.1.18',
        'py>=1.4.20',
        'pytest>=2.5.2',
        'moto>=0.3.9',
    ]
}


setup(
    name='Pontus',
    version=get_version(),
    url='https://github.com/fastmonkeys/pontus',
    author='Vesa Uimonen',
    author_email='vesa@fastmonkeys.com',
    description='Flask utility for Amazon S3.',
    license='MIT',
    packages=['pontus'],
    zip_safe=False,
    include_package_data=True,
    platforms='any',
    install_requires=[
        'Flask>=0.10.1',
        'python-magic>=0.4.6',
        'boto3>=1.4.7'
    ],
    extras_require=extras_require,
    cmdclass={'test': PyTest},
    classifiers=[
        'Environment :: Web Environment',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Topic :: Internet :: WWW/HTTP :: Dynamic Content',
        'Topic :: Software Development :: Libraries :: Python Modules'
    ]
)



# -*- coding: utf-8 -*-
import base64
import hmac
import json
import uuid
from datetime import datetime, timedelta
from hashlib import sha1

from flask import current_app

from ._compat import force_bytes, force_text, unicode_compatible


@unicode_compatible
class AmazonS3SignedRequest(object):
    """A Flask utility for creating signatures for
    `POST requests to Amazon S3`_.

    .. _POST requests to Amazon S3:
        http://docs.aws.amazon.com/AmazonS3/latest/API/sigv4-authentication-HTTPPOST.html

    If the application has an `AWS_UNVALIDATED_PREFIX` config, its value
    will be added to the file key.

    Example::

        import boto
        from pontus import AmazonS3SignedRequest

        connection = boto.s3.connection.S3Connection()
        bucket = connection.get_bucket('testbucket')

        signed_request = AmazonS3SignedRequest(
            key_name=u'my/file.jpg',
            mime_type=u'image/jpeg',
            bucket=bucket,
        )

        assert signed_request.form_fields == {
            'AWSAccessKeyId': 'your-aws-access-key',
            'acl': 'public-read',
            'key': 'my/file.jpg',
            'Policy': 'generated-policy-document',
            'success_action_status': '201',
            'Signature': 'generated-signature',
        }

    :param key_name:
        The key name of the file to be stored in Amazon S3.

    :param mime_type:
        The MIME type of the file to be stored in Amazon S3.

    :param bucket:
        The Boto Bucket instance to be used.

    :param acl:
        The ACL of the uploaded file, for example 'public-read' or 'private'.

    :param expires_in:
        The expiry time of the signature.

    :param success_action_status:
        Status code Amazon S3 should respond with on successful POST.

    :param max_content_length:
        The maximum length of the file to be stored.

    :param randomize:
        Indicates if a randomized UUID prefix should be added to the file key.
    """
    def __init__(
        self,
        key_name,
        mime_type,
        bucket,
        session,
        acl='public-read',
        expires_in=60,
        success_action_status='201',
        max_content_length=None,
        randomize=False,
    ):
        if randomize:
            key_name = u'%s/%s' % (uuid.uuid4(), key_name)

        key_name = u'%s%s' % (
            current_app.config.get('AWS_UNVALIDATED_PREFIX', ''),
            key_name
        )

        self.expires_in = expires_in
        self.key_name = key_name
        self.acl = acl
        self.max_content_length = (
            max_content_length or
            current_app.config.get('MAX_CONTENT_LENGTH') or
            20971520
        )
        self.mime_type = mime_type
        self.randomize = randomize
        self.bucket = bucket
        self.session = session
        self.success_action_status = success_action_status

    @property
    def form_fields(self):
        """
        Generates form fields needed for creating a signed POST request to
        Amazon S3.

        :return: a dictionary containing the needed field values
        """
        policy = self._get_policy_document()
        return {
            'AWSAccessKeyId': self.session.get_credentials().access_key,
            'acl': self.acl,
            'key': self.key_name,
            'Policy': policy,
            'success_action_status': self.success_action_status,
            'Signature': self._get_signature(policy),
        }

    def _get_policy_document(self):
        expiration = datetime.utcnow() + timedelta(seconds=self.expires_in)
        data = {
            'expiration': expiration.isoformat() + 'Z',
            'conditions': [
                {'bucket': self.bucket.name},
                {'key': self.key_name},
                {'acl': self.acl},
                ['starts-with', '$Content-Type', ''],
                ['content-length-range', 0, self.max_content_length],
                {'success_action_status': self.success_action_status}
            ]
        }
        data = json.dumps(data)
        return force_text(base64.b64encode(force_bytes(data)))

    def _get_signature(self, policy_document):
        signature = base64.encodestring(hmac.new(
            force_bytes(self.session.get_credentials().secret_key),
            force_bytes(policy_document),
            sha1
        ).digest()).strip()
        return force_text(signature)

    def __repr__(self):
        return "<{cls} key_name='{key_name!s}'>".format(
            cls=self.__class__.__name__,
            key_name=self.key_name
        )

    def __str__(self):
        return self.key_name



# -*- coding: utf-8 -*-
import botocore
from flask import current_app

from .exceptions import FileNotFoundError, ValidationError


class AmazonS3FileValidator(object):
    """A Flask utility for validating files stored in AmazonS3.

    If the application has an `AWS_UNVALIDATED_PREFIX` config, its value
    will be removed from the file key if the file is valid. This enables
    deleting unvalidated files via `Amazon S3 Lifecycle Management`_.

    .. _Amazon S3 Lifecycle Management:
        http://docs.aws.amazon.com/AmazonS3/latest/dev/object-lifecycle-mgmt.html

    Example::

        import boto
        from pontus import AmazonS3FileValidator
        from pontus.exceptions import FileNotFoundError
        from pontus.validators import MimeType

        connection = boto.s3.connection.S3Connection()
        bucket = connection.get_bucket('testbucket')

        try:
            validator = AmazonS3FileValidator(
                key_name='my/file.jpg',
                bucket=bucket,
                validators=[MimeType('image/jpeg')]
            )
        except FileNotFoundError:
            # File was not found
            pass

        if validator.validate():
            # File MIME type was image/jpeg
            pass
        else:
            # File was invalid, printing errors
            print validator.errors

    :param key_name:
        The key of the file stored in Amazon S3.

    :param bucket:
        The Boto S3 Bucket instance.

    :param validators:
        List of validators. A validator can either be an instance of a class
        inheriting :class:`BaseValidator` or a callable
        function that takes Boto S3 Object instance as a parameter.

    """
    def __init__(self, key_name, bucket, validators=[]):
        self.errors = []
        self.obj = bucket.Object(key_name)
        try:
            self.obj.load()
        except botocore.exceptions.ClientError as e:
            if e.response['Error']['Code'] == '404':
                raise FileNotFoundError(key=key_name)
            else:
                raise e
        self.bucket = bucket
        self.validators = validators

    def validate(self):
        """
        Validates the given Amazon S3 file with :attr:`validators`. If errors
        occur they are appended to :attr:`errors`. If the file is valid and a
        `AWS_UNVALIDATED_PREFIX` config is present, its value will be removed
        from the file key.

        :return: a boolean indicating if the file vas valid.
        """
        for validator in self.validators:
            try:
                validator(self.obj)
            except ValidationError as e:
                self.errors.append(e.error)

        if not self.errors and self._has_unvalidated_prefix():
            self._move_to_validated()

        return not self.errors

    def _has_unvalidated_prefix(self):
        return (
            current_app.config.get('AWS_UNVALIDATED_PREFIX') and
            self.obj.key.startswith(
                current_app.config.get('AWS_UNVALIDATED_PREFIX')
            )
        )

    def _move_to_validated(self):
        new_name = self.obj.key[
            len(current_app.config.get('AWS_UNVALIDATED_PREFIX')):
        ]
        new_obj = self.bucket.Object(new_name)
        new_obj.copy({
            'Bucket': self.bucket.name,
            'Key': self.obj.key
        })
        new_obj.Acl().put(ACL='public-read')
        self.obj.delete()
        self.obj = new_obj

    def __repr__(self):
        return '<{cls} key={key!r}>'.format(
            cls=self.__class__.__name__,
            key=self.obj.key
        )



import sys

is_py3 = sys.version_info[0] > 2

if is_py3:
    binary_type = bytes
    text_type = str
else:
    binary_type = str
    text_type = unicode


def force_text(s, encoding='utf-8'):
    if isinstance(s, text_type):
        return s
    return s.decode(encoding)


def force_bytes(s, encoding='utf-8'):
    if isinstance(s, binary_type):
        return s
    return s.encode(encoding)


def unicode_compatible(cls):
    """
    A decorator that defines ``__str__`` and ``__unicode__`` methods
    under Python 2.
    """
    if not is_py3:
        cls.__unicode__ = cls.__str__
        cls.__str__ = lambda self: self.__unicode__().encode('utf-8')
    return cls



# -*- coding: utf-8 -*-
import base64
import json

import boto3
import freezegun
import pytest
from flexmock import flexmock

from pontus import AmazonS3SignedRequest
from pontus._compat import force_text

HOUR_IN_SECONDS = 60 * 60


class TestAmazonS3SignedRequest(object):
    @pytest.fixture
    def signed_request(self, bucket):
        return AmazonS3SignedRequest(
            key_name='file_name.png',
            mime_type='image/png',
            bucket=bucket,
            acl='private',
            expires_in=HOUR_IN_SECONDS,
            session=boto3.session.Session(
                aws_access_key_id='test-key',
                aws_secret_access_key='test-secret-key'
            )
        )

    def test_key(self, signed_request):
        assert signed_request.key_name == (
            'test-unvalidated-uploads/file_name.png'
        )

    def test_get_signature(self, signed_request):
        signature = signed_request._get_signature('some policy')
        assert signature == '+kyVfzakuUYOwB4mwo2CuG5GiGI='

    def test_get_policy_document(self, signed_request):
        with freezegun.freeze_time('2007-12-01 12:05:37.572123'):
            policy = signed_request._get_policy_document()
        data_as_json = base64.b64decode(policy)
        data = json.loads(force_text(data_as_json))
        assert data == {
            'expiration': '2007-12-01T13:05:37.572123Z',
            'conditions': [
                {'bucket': 'test-bucket'},
                {'key': 'test-unvalidated-uploads/file_name.png'},
                {'acl': 'private'},
                ['starts-with', '$Content-Type', ''],
                ['content-length-range', 0, 20971520],
                {'success_action_status': '201'},
            ]
        }

    def test_form_fields(self, signed_request):
        (
            flexmock(signed_request)
            .should_receive('_get_policy_document')
            .and_return(u'policy')
        )
        (
            flexmock(signed_request)
            .should_receive('_get_signature')
            .and_return(u'signature')
        )
        assert signed_request.form_fields == {
            'AWSAccessKeyId': 'test-key',
            'acl': 'private',
            'key': 'test-unvalidated-uploads/file_name.png',
            'Policy': 'policy',
            'success_action_status': '201',
            'Signature': 'signature',
        }

    def test_repr(self, signed_request):
        assert repr(signed_request) == (
            u"<AmazonS3SignedRequest " +
            u"key_name='test-unvalidated-uploads/file_name.png'>"
        )

    def test_randomized_key(self, bucket):
        import uuid
        (
            flexmock(uuid)
            .should_receive('uuid4')
            .and_return(u'random-string')
        )
        signed_request = AmazonS3SignedRequest(
            key_name='file_name.png',
            mime_type='image/png',
            randomize=True,
            bucket=bucket,
            session=boto3.session.Session()
        )
        assert signed_request.key_name == (
            'test-unvalidated-uploads/random-string/file_name.png'
        )
